//+------------------------------------------------------------------+
//|                                                      mvt_Vlt.mq4 |
//|                   Copyright 2005-2014, MetaQuotes Software Corp. |
//|                                              http://www.mql4.com |
//+------------------------------------------------------------------+
#property copyright   "2005-2014, MetaQuotes Software Corp."
#property link        "http://www.mql4.com"
#property description "mvt_Vlt"
#property strict

#include <MovingAverages.mqh>

#property indicator_separate_window//indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 clrRed
#property indicator_color2 clrLawnGreen
#property indicator_color3 clrYellow
//--- indicator parameters
input int   InpTypeVlt=0;           
input int   InpBandsPeriod=20;      
//--- buffers
double ExtCalcVlBuffer[];  //  ,       
double ExtMovingBuffer[];
double ExtStdDevBuffer[];
//
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//--- 
   IndicatorBuffers(3);
   IndicatorDigits(Digits);
//--- middle line
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,ExtMovingBuffer);
   SetIndexLabel(0,"vlt_av");
//--- 
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,ExtStdDevBuffer);
   SetIndexLabel(1,"vlt_std");
//--- 
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,ExtCalcVlBuffer); 
   SetIndexLabel(2,"CandleHight");
//   
//--- check for input parameter
   if(InpBandsPeriod<=0)
     {
      Print("Wrong input parameter Bands Period=",InpBandsPeriod);
      return(INIT_FAILED);
     }
//---
   SetIndexDrawBegin(0,InpBandsPeriod);
   SetIndexDrawBegin(1,InpBandsPeriod);
   SetIndexDrawBegin(2,InpBandsPeriod);

//--- initialization done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| mvt_Vlt
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   int i,pos;
//---
   if(rates_total<=InpBandsPeriod || InpBandsPeriod<=0)
      return(0);
//--- counting from 0 to rates_total
   ArraySetAsSeries(ExtCalcVlBuffer,false); 
   ArraySetAsSeries(ExtMovingBuffer,false);
   ArraySetAsSeries(ExtStdDevBuffer,false);
   ArraySetAsSeries(high,false);             
   ArraySetAsSeries(low,false);              
// 
   ArraySetAsSeries(close,false);
   ArraySetAsSeries(open,false);             
   
//--- initial zero
   if(prev_calculated<1)
     {
      for(i=0; i<InpBandsPeriod; i++)
        {
         ExtCalcVlBuffer[i]=EMPTY_VALUE;
         ExtMovingBuffer[i]=EMPTY_VALUE;
         ExtStdDevBuffer[i]=EMPTY_VALUE;
        }
     }
//--- starting calculation
   if(prev_calculated>1)
      pos=prev_calculated-1;
   else
      pos=0;
//--- main cycle
    for(i=pos; i<rates_total   && !IsStopped(); i++)     //     
    //for(i=pos; i<rates_total-1 && !IsStopped(); i++)     //      
     {
      
         ExtCalcVlBuffer[i]=high[i]-low[i];
         ExtMovingBuffer[i]=SimpleMA(i,InpBandsPeriod,ExtCalcVlBuffer);
         ExtStdDevBuffer[i]=StdDev_Func(i,ExtCalcVlBuffer,ExtMovingBuffer,InpBandsPeriod);
 
     }
//---- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Calculate Standard Deviation                                     |
//+------------------------------------------------------------------+
double StdDev_Func(int position,const double &price[],const double &MAprice[],int period)
  {
//--- variables
   double StdDev_dTmp=0.0;
//--- check for position
   if(position>=period)
     {
      //--- calcualte StdDev
      for(int i=0; i<period; i++)
         StdDev_dTmp+=MathPow(price[position-i]-MAprice[position],2);
      StdDev_dTmp=MathSqrt(StdDev_dTmp/(period-1));
     }
//--- return calculated value
   return(StdDev_dTmp);
  }
//+------------------------------------------------------------------+
